import rclpy
from rclpy.node import Node
import numpy as np
from std_msgs.msg import Float64
from geometry_msgs.msg import Point, PoseStamped
import time


class Reference_Node_1DoF(Node):

    def __init__(self):
        super().__init__('reference_node_1dof')
        #Create a publisher sending the vector as a message
        self.publisher_ = self.create_publisher(Float64, '/dof1/reference_angles', 10)

        self.declare_parameter('velocity', 0.0)
        self.declare_parameter('acceleration', 0.0)
        self.declare_parameter('decelaration', 0.0)
        self.declare_parameter('velocity_constant', 0.0)
        self.declare_parameter('slope', 0.0)
        self.declare_parameter('constant_speed', False)

        self.velocity = self.get_parameter('velocity').get_parameter_value().double_value
        self.acceleration = self.get_parameter('acceleration').get_parameter_value().double_value
        self.decelaration = self.get_parameter('decelaration').get_parameter_value().double_value
        self.velocity_constant = self.get_parameter('velocity_constant').get_parameter_value().double_value
        self.slope = self.get_parameter('slope').get_parameter_value().double_value
        self.constant_speed = self.get_parameter('constant_speed').get_parameter_value().bool_value

        self.phi_start = 0.0
        self.phi_end = 1.0
        self.phi = self.phi_start
        self.velocity_start = self.velocity

        self.timer_period = 0.1  # seconds
        self.timer = self.create_timer(self.timer_period, self.ref_point_publisher)
        # List the points
        
    def ref_point_publisher(self):
        # Ref position over the time
        # Interpolate between x_0 - x_1
               
        msg = Float64()

        if self.constant_speed == True:
            self.phi = self.phi + self.velocity_constant  
        elif self.constant_speed == False:
            if self.phi < self.slope:
                self.phi = self.phi + self.velocity
                self.velocity = self.velocity + self.acceleration
                self.get_logger().info('Acceleration phase')
                self.get_logger().info('%f' % self.velocity)
            elif self.phi > 1 - self.slope:
                self.phi = self.phi + self.velocity
                self.velocity = self.velocity - self.decelaration
                self.get_logger().info('Decelaration phase')
                self.get_logger().info('%f' % self.velocity)
            else:
                self.phi = self.phi + self.velocity
                self.get_logger().info('Constant speed phase')
                self.get_logger().info('%f' % self.velocity)

        #setting back to 0
        if self.phi>self.phi_end:
            self.phi = self.phi_start
            self.velocity = self.velocity_start
            # self.get_logger().info('Resetting "%f"' % self.phi)

        msg.data = self.phi
        self.publisher_.publish(msg)
        

def main(args = None):
    rclpy.init(args = args)
    node = Reference_Node_1DoF()
    rclpy.spin(node)
    node.destroy_node()
    rclpy.shutdown()

if __name__ == '__main__':
    main() 